@@ -43,6 +43,10 @@ namespace MonoDevelop.Debugger.Mdb
 			get { return "Mono Debugger"; }
 		}
 		
+		public string Id {
+			get { return "Mono.Debugger.Mdb"; }
+		}
+		
 		public bool CanDebugCommand (ExecutionCommand command)
 		{
 			DotNetExecutionCommand cmd = command as DotNetExecutionCommand;
@@ -80,7 +84,7 @@ namespace MonoDevelop.Debugger.Mdb
 			return ds;
 		}
 		
-		public ProcessInfo[] GetAttachablePocesses ()
+		public ProcessInfo[] GetAttachableProcesses ()
 		{
 			List<ProcessInfo> procs = new List<ProcessInfo> ();
 			foreach (string dir in Directory.GetDirectories ("/proc")) {
