@@ -43,30 +43,63 @@ namespace DebuggerServer
 	{
 		public override string CallToString (EvaluationContext gctx, object obj)
 		{
-			if (!gctx.AllowTargetInvoke)
+			if (!gctx.Options.AllowTargetInvoke)
 				return GetValueTypeName (gctx, obj);
 			MdbEvaluationContext ctx = (MdbEvaluationContext) gctx;
+			obj = ctx.GetRealObject (obj);
+			if (obj is TargetFundamentalObject) {
+				object fob = ((TargetFundamentalObject)obj).GetObject (ctx.Thread);
+				return fob != null ? fob.ToString () : null;
+			}
+			
+			TargetStructObject starget = obj as TargetStructObject;
+			if (starget != null) {
+				// Optimization: if ToString is not overriden, there is no need to call it since the result is the name of the type
+				MemberReference mem = OverloadResolve (ctx, "ToString", starget.Type, new TargetType [0], true, false, false);
+				if (mem == null || mem.DeclaringType.Name == "System.Object")
+					return GetDisplayTypeName (ctx, starget.Type);
+			}
+			
 			TargetObject retval = CallMethod (ctx, "ToString", (TargetStructObject) obj);
 			object s = ((TargetFundamentalObject) retval).GetObject (ctx.Thread);
 			return s != null ? s.ToString () : "";
 		}
 
-		public override object Cast (EvaluationContext gctx, object obj, object targetType)
-		{
-			MdbEvaluationContext ctx = (MdbEvaluationContext) gctx;
-			return TargetObjectConvert.Cast (ctx, (TargetObject) obj, (TargetType) targetType);
-		}
-
-		public override object TryCast (EvaluationContext gctx, object val, object type)
+		public override object TryCast (EvaluationContext gctx, object val, object targetType)
 		{
 			MdbEvaluationContext ctx = (MdbEvaluationContext)gctx;
 			TargetObject tval = ctx.GetRealObject (val);
-			if (type is TargetObjectType)
+			if (targetType is TargetObjectType)
 				return val;
-			if (type is TargetClassType)
-				return TargetObjectConvert.TryCast (ctx, tval, (TargetClassType)type);
-			else
-				return null;
+			else if (targetType is TargetClassType)
+				return TargetObjectConvert.TryCast (ctx, tval, (TargetClassType)targetType);
+			else if (targetType is TargetEnumType) {
+				TargetType tt = ctx.Frame.Language.LookupType ("System.Enum");
+				TargetObject typeObj = (TargetObject) GetTypeOf (ctx, ((TargetEnumType)targetType).Name);
+				return CallStaticMethod (ctx, "ToObject", tt, typeObj, tval);
+			}
+			else if (targetType is TargetFundamentalType) {
+				if (tval is TargetEnumObject) {
+					TargetEnumObject eob = (TargetEnumObject) tval;
+					return TryCast (ctx, eob.GetValue (ctx.Thread), targetType);
+				}
+				TargetFundamentalType ftype = (TargetFundamentalType) targetType;
+				Type tt = Type.GetType (ObjectUtil.FixTypeName (ftype.Name));
+				if (tval is TargetFundamentalObject) {
+					TargetFundamentalObject fob = (TargetFundamentalObject) tval;
+					object obj = fob.GetObject (ctx.Thread);
+					if (tt.IsInstanceOfType (obj))
+						return val;
+					if (tt != typeof(string) && !(obj is string)) {
+						try {
+							object res = System.Convert.ChangeType (obj, tt);
+							return CreateValue (ctx, res);
+						} catch {
+						}
+					}
+				}
+			}
+			return null;
 		}
 
 
@@ -76,48 +109,12 @@ namespace DebuggerServer
 			TargetObject obj = ctx.GetRealObject (vobj);
 			
 			switch (obj.Kind) {
-				case Mono.Debugger.Languages.TargetObjectKind.Array:
-					TargetArrayObject arr = obj as TargetArrayObject;
-					if (arr == null)
-						return null;
-					StringBuilder tn = new StringBuilder (arr.Type.ElementType.Name);
-					tn.Append ("[");
-					TargetArrayBounds ab = arr.GetArrayBounds (ctx.Thread);
-					if (ab.IsMultiDimensional) {
-						for (int n=0; n<ab.Rank; n++) {
-							if (n>0)
-								tn.Append (',');
-							tn.Append (ab.UpperBounds [n] - ab.LowerBounds [n] + 1);
-						}
-					}
-					else if (!ab.IsUnbound) {
-						tn.Append (ab.Length.ToString ());
-					}
-					tn.Append ("]");
-					return new LiteralExp (tn.ToString ());
-					
+				case TargetObjectKind.Array:
 				case TargetObjectKind.GenericInstance:
 				case TargetObjectKind.Struct:
 				case TargetObjectKind.Class:
-					TypeDisplayData tdata = GetTypeDisplayData (ctx, obj.Type);
-					TargetStructObject co = obj as TargetStructObject;
-					if (co == null)
-						return null;
-					if (ctx.AllowTargetInvoke) {
-						if (co.TypeName == "System.Decimal")
-							return new LiteralExp (CallToString (ctx, co));
-						if (tdata.ValueDisplayString != null)
-							return new LiteralExp (EvaluateDisplayString (ctx, co, tdata.ValueDisplayString));
-					}
-					
-					// Return the type name
-					if (tdata.TypeDisplayString != null)
-						return new LiteralExp ("{" + tdata.TypeDisplayString + "}");
-					return new LiteralExp ("{" + obj.Type.Name + "}");
-					
 				case TargetObjectKind.Enum:
-					TargetEnumObject eob = (TargetEnumObject) obj;
-					return new LiteralExp (Server.Instance.Evaluator.TargetObjectToString (ctx, eob.GetValue (ctx.Thread)));
+					return base.TargetObjectToObject (ctx, vobj);
 					
 				case TargetObjectKind.Fundamental:
 					TargetFundamentalObject fob = obj as TargetFundamentalObject;
@@ -136,7 +133,7 @@ namespace DebuggerServer
 					if (oob == null)
 						return null;
 					else
-						return new LiteralExp ("{" + oob.TypeName + "}");
+						return new EvaluationResult ("{" + oob.TypeName + "}");
 					
 				case TargetObjectKind.Nullable:
 					TargetNullableObject nob = (TargetNullableObject) obj;
@@ -145,7 +142,7 @@ namespace DebuggerServer
 					else
 						return null;
 			}
-			return new LiteralExp ("?");
+			return new EvaluationResult ("?");
 		}
 
 
@@ -194,7 +191,6 @@ namespace DebuggerServer
 				TypeDisplayData data = new TypeDisplayData ();
 				if (tt.ClassType.DebuggerTypeProxyAttribute != null) {
 					data.ProxyType = tt.ClassType.DebuggerTypeProxyAttribute.ProxyTypeName;
-					data.IsProxyType = true;
 				}
 				if (tt.ClassType.DebuggerDisplayAttribute != null) {
 					data.NameDisplayString = tt.ClassType.DebuggerDisplayAttribute.Name;
@@ -219,6 +215,12 @@ namespace DebuggerServer
 			return ctx.GetRealObject (val) is TargetFundamentalObject;
 		}
 
+		public override bool IsEnum (EvaluationContext gctx, object val)
+		{
+			MdbEvaluationContext ctx = (MdbEvaluationContext) gctx;
+			return ctx.GetRealObject (val) is TargetEnumObject;
+		}
+
 		public override bool IsNull (EvaluationContext gctx, object val)
 		{
 			MdbEvaluationContext ctx = (MdbEvaluationContext) gctx;
@@ -268,6 +270,15 @@ namespace DebuggerServer
 			return mctx.Frame.Language.LookupType (name);
 		}
 
+		public override object GetBaseType (EvaluationContext ctx, object t)
+		{
+			MdbEvaluationContext mctx = (MdbEvaluationContext) ctx;
+			TargetStructType type = t as TargetStructType;
+			if (type != null && type.HasParent)
+				return type.GetParentType (mctx.Thread);
+			else
+				return null;
+		}
 
 		public override object[] GetTypeArgs (EvaluationContext ctx, object type)
 		{
@@ -409,7 +420,7 @@ namespace DebuggerServer
 			return obj.GetType ().GetProperty (name).GetValue (obj, null);
 		}
 
-		public override IEnumerable<ValueReference> GetMembers (EvaluationContext gctx, object tt, object ob, BindingFlags bindingFlags)
+		protected override IEnumerable<ValueReference> GetMembers (EvaluationContext gctx, object tt, object ob, BindingFlags bindingFlags)
 		{
 			MdbEvaluationContext ctx = (MdbEvaluationContext)gctx;
 			TargetStructObject co = ctx.GetRealObject (ob) as TargetStructObject; // It can be a TargetObjectObject
@@ -423,7 +434,7 @@ namespace DebuggerServer
 					TargetFieldInfo field = (TargetFieldInfo)mem.Member;
 					yield return new FieldReference (ctx, co, mem.DeclaringType, field);
 				}
-				if (mem.Member is TargetPropertyInfo && gctx.AllowTargetInvoke) {
+				if (mem.Member is TargetPropertyInfo) {
 					TargetPropertyInfo prop = (TargetPropertyInfo) mem.Member;
 					if (prop.CanRead && (prop.Getter.ParameterTypes == null || prop.Getter.ParameterTypes.Length == 0))
 						yield return new PropertyReference (ctx, prop, co);
@@ -511,7 +522,7 @@ namespace DebuggerServer
 			TargetObject obj = ctx.GetRealObject (vobj);
 			
 			if (obj == null)
-				return ObjectValue.CreateObject (null, path, "", null, flags | ObjectValueFlags.ReadOnly, null);
+				return ObjectValue.CreateObject (null, path, "", "", flags | ObjectValueFlags.ReadOnly, null);
 
 			if (obj.HasAddress && obj.GetAddress (ctx.Thread).IsNull)
 				return ObjectValue.CreateObject (null, path, obj.TypeName, ctx.Evaluator.ToExpression (null), flags, null);
@@ -521,29 +532,10 @@ namespace DebuggerServer
 				case TargetObjectKind.Struct:
 				case TargetObjectKind.GenericInstance:
 				case TargetObjectKind.Class:
-					TypeDisplayData tdata = GetTypeDisplayData (ctx, obj.Type);
-					
-					TargetStructObject co = obj as TargetStructObject;
-					if (co == null)
-						return ObjectValue.CreateUnknown (path.LastName);
-					else {
-						string tvalue;
-						if (!string.IsNullOrEmpty (tdata.ValueDisplayString))
-							tvalue = EvaluateDisplayString (ctx, co, tdata.ValueDisplayString);
-						else
-							tvalue = ctx.Evaluator.TargetObjectToExpression (ctx, obj);
-						
-						string tname;
-						if (!string.IsNullOrEmpty (tdata.TypeDisplayString))
-							tname = EvaluateDisplayString (ctx, co, tdata.TypeDisplayString);
-						else
-							tname = obj.TypeName;
-						
-						ObjectValue val = ObjectValue.CreateObject (source, path, tname, tvalue, flags, null);
-						if (!string.IsNullOrEmpty (tdata.NameDisplayString))
-							val.Name = EvaluateDisplayString (ctx, co, tdata.NameDisplayString);
-						return val;
-					}
+				case TargetObjectKind.Array:
+				case TargetObjectKind.Fundamental:
+				case TargetObjectKind.Enum:
+					return base.CreateObjectValueImpl (gctx, source, path, vobj, flags);
 					
 				case TargetObjectKind.Object:
 					TargetObjectObject oob = obj as TargetObjectObject;
@@ -552,18 +544,6 @@ namespace DebuggerServer
 					else
 						return ObjectValue.CreateObject (source, path, obj.TypeName, Server.Instance.Evaluator.TargetObjectToExpression (ctx, obj), flags, null);
 					
-				case TargetObjectKind.Array:
-					return ObjectValue.CreateObject (source, path, obj.TypeName, Server.Instance.Evaluator.TargetObjectToExpression (ctx, obj), flags, null);
-					
-				case TargetObjectKind.Fundamental:
-					TargetFundamentalObject fob = (TargetFundamentalObject) obj;
-					return ObjectValue.CreatePrimitive (source, path, obj.TypeName, Server.Instance.Evaluator.TargetObjectToExpression (ctx, fob), flags);
-					
-				case TargetObjectKind.Enum:
-					Console.WriteLine ("pp ??? ENUM: " + obj.GetType ());
-					TargetEnumObject enumobj = (TargetEnumObject) obj;
-					return CreateObjectValue (ctx, source, path, enumobj.GetValue (ctx.Thread), flags);
-					
 				case TargetObjectKind.Pointer:
 					return ObjectValue.CreateObject (source, path, obj.TypeName, Server.Instance.Evaluator.TargetObjectToExpression (ctx, obj), flags, null);
 					
@@ -579,7 +559,7 @@ namespace DebuggerServer
 						return ObjectValue.CreateObject (source, path, obj.TypeName, ctx.Evaluator.ToExpression (null), flags, new ObjectValue [0]);
 					}
 				default:
-					return ObjectValue.CreateError (path.LastName, "Unknown value type: " + obj.Kind, flags);
+					return ObjectValue.CreateFatalError (path.LastName, "Unknown value type: " + obj.Kind, flags);
 			}
 		}
 
@@ -629,10 +609,10 @@ namespace DebuggerServer
 			MemberReference mem = OverloadResolve (ctx, name, starget.Type, types, true, false, true);
 			TargetFunctionType function = (TargetFunctionType) ((TargetMethodInfo) mem.Member).Type;
 			
-			while (target.Type != mem.DeclaringType) {
+			while (starget.Type != mem.DeclaringType) {
 				TargetStructObject par = starget.GetParentObject (ctx.Thread);
 				if (par != null)
-					target = par;
+					starget = par;
 				else
 					break;
 			}
@@ -674,7 +654,7 @@ namespace DebuggerServer
 			for (int n=0; n<types.Length; n++)
 				types [n] = args [n].Type;
 			
-			MemberReference mem = OverloadResolve (ctx, name, (TargetStructType) type, types, false, true, true);
+			MemberReference mem = OverloadResolve (ctx, name, type, types, false, true, true);
 			TargetFunctionType function = (TargetFunctionType) ((TargetMethodInfo) mem.Member).Type;
 			
 			TargetMethodSignature sig = function.GetSignature (ctx.Thread);
@@ -687,7 +667,7 @@ namespace DebuggerServer
 			return Server.Instance.RuntimeInvoke (ctx, function, null, objs);
 		}
 		
-		public static MemberReference OverloadResolve (MdbEvaluationContext ctx, string methodName, TargetStructType type, TargetType[] argtypes, bool allowInstance, bool allowStatic, bool throwIfNotFound)
+		public static MemberReference OverloadResolve (MdbEvaluationContext ctx, string methodName, TargetType type, TargetType[] argtypes, bool allowInstance, bool allowStatic, bool throwIfNotFound)
 		{
 			List<MemberReference> candidates = new List<MemberReference> ();
 
@@ -833,10 +813,6 @@ namespace DebuggerServer
 						TargetObject res = co.GetCurrentObject (ctx.Thread);
 						return res ?? obj;
 						
-					case TargetObjectKind.Enum:
-						TargetEnumObject eob = (TargetEnumObject) obj;
-						return eob.GetValue (ctx.Thread);
-						
 					case TargetObjectKind.Object:
 						TargetObjectObject oob = obj as TargetObjectObject;
 						if (oob == null)
@@ -859,16 +835,15 @@ namespace DebuggerServer
 			// must be retrieved before we can do anything with them.
 			List<MemberReference> members = new List<MemberReference> ();
 			
-			TargetStructType type = t as TargetStructType;
 			Dictionary<string,string> foundMethods = new Dictionary<string,string> ();
 
-			while (type != null) {
+			while (t != null) {
 				
 				TargetFieldInfo[] fields = null;
 				TargetPropertyInfo[] properties = null;
 				TargetMethodInfo[] methods = null;
 
-				TargetClass cls = type.GetClass (ctx.Thread);
+				TargetClass cls = t.HasClassType ? t.ClassType.GetClass (ctx.Thread) : null;
 				if (cls != null) {
 					if (includeFields)
 						fields = cls.GetFields (ctx.Thread);
@@ -878,9 +853,9 @@ namespace DebuggerServer
 						methods = cls.GetMethods (ctx.Thread);
 				}
 				else {
-					TargetClassType ct = type as TargetClassType;
-					if (ct == null && type.HasClassType)
-						ct = type.ClassType;
+					TargetClassType ct = t as TargetClassType;
+					if (ct == null && t.HasClassType)
+						ct = t.ClassType;
 					if (ct != null) {
 						if (includeFields)
 							fields = ct.Fields;
@@ -903,7 +878,7 @@ namespace DebuggerServer
 							continue;
 						if (!field.IsStatic && (flags & BindingFlags.Instance) == 0)
 							continue;
-						members.Add (new MemberReference (field, type));
+						members.Add (new MemberReference (field, t));
 					}
 				}
 				
@@ -917,7 +892,7 @@ namespace DebuggerServer
 							continue;
 						if (!prop.IsStatic && (flags & BindingFlags.Instance) == 0)
 							continue;
-						members.Add (new MemberReference (prop, type));
+						members.Add (new MemberReference (prop, t));
 					}
 				}
 				
@@ -934,13 +909,14 @@ namespace DebuggerServer
 						string sig = met.FullName;
 						if (!foundMethods.ContainsKey (sig)) {
 							foundMethods [sig] = sig;
-							members.Add (new MemberReference (met, type));
+							members.Add (new MemberReference (met, t));
 						}
 					}
 				}
 				
-				if (type.HasParent)
-					type = type.GetParentType (ctx.Thread);
+				TargetStructType type = t as TargetStructType;
+				if (type != null && type.HasParent && (flags & BindingFlags.DeclaredOnly) == 0)
+					t = type.GetParentType (ctx.Thread);
 				else
 					break;
 			}
@@ -998,6 +974,7 @@ namespace DebuggerServer
 				
 				sb.Remove (sb.Length - 1, 1);
 				sb.Append (']');
+				sb.Append (typeName.Substring (e+1));
 				return sb.ToString ();
 			}
 			
@@ -1028,9 +1005,9 @@ namespace DebuggerServer
 	public class MemberReference
 	{
 		public readonly TargetMemberInfo Member;
-		public readonly TargetStructType DeclaringType;
+		public readonly TargetType DeclaringType;
 		
-		public MemberReference (TargetMemberInfo member, TargetStructType type)
+		public MemberReference (TargetMemberInfo member, TargetType type)
 		{
 			Member = member;
 			DeclaringType = type;
@@ -1046,7 +1023,7 @@ namespace DebuggerServer
 				TargetFieldInfo field = (TargetFieldInfo) Member;
 				if (field.HasConstValue)
 					return ctx.Frame.Language.CreateInstance (ctx.Thread, field.ConstValue);
-				TargetClass cls = DeclaringType.GetClass (ctx.Thread);
+				TargetClass cls = DeclaringType.ClassType.GetClass (ctx.Thread);
 				return ObjectUtil.GetRealObject (ctx, cls.GetField (ctx.Thread, thisObj, field));
 			}
 			else {
@@ -1071,6 +1048,15 @@ namespace DebuggerServer
 			this.object_argument = object_argument;
 			this.param_objects = param_objects;
 		}
+		
+		public override string Description {
+			get {
+				if (function.DeclaringType != null)
+					return function.DeclaringType.Name + "." + function.Name;
+				else
+					return function.Name;
+			}
+		}
 
 		public override void Invoke ( )
 		{
@@ -1084,11 +1070,19 @@ namespace DebuggerServer
 			Server.Instance.MdbAdaptor.AbortThread (ctx.Thread, res);
 			WaitToStop (ctx.Thread);
 		}
+		
+		public override void Shutdown ()
+		{
+			res.Abort ();
+			if (!res.CompletedEvent.WaitOne (200))
+				return;
+			Server.Instance.MdbAdaptor.AbortThread (ctx.Thread, res);
+		}
 
 		public override bool WaitForCompleted (int timeout)
 		{
-			if (ctx.Timeout != -1) {
-				if (!res.CompletedEvent.WaitOne (ctx.Timeout, false))
+			if (timeout != -1) {
+				if (!res.CompletedEvent.WaitOne (timeout, false))
 					return false;
 			}
 			else {
